@@ -3,7 +3,7 @@ pesudo_sequence <- seq(-5, 5, length = 2)
 two_dim_location <-
   as.matrix(expand.grid(x = pesudo_sequence, y = pesudo_sequence))
 
-three_dim_location <- 
+three_dim_location <-
   as.matrix(expand.grid(x = pesudo_sequence, y = pesudo_sequence, z = pesudo_sequence))
 
 # thinPlateMatrix
@@ -18,8 +18,6 @@ test_that("Thin-Plate Spline Matrix", {
 Phi <- matrix(c(1, 0, 0, 0), nrow = 4, ncol = 1)
 new_location <- matrix(c(0.1, 0.2), nrow = 1, ncol = 2)
 test_that("Eigen-function", {
-  expect_lte(
-    eigenFunction(new_location, two_dim_location, Phi) - 0.2352884,
-    tol
-  )
+  expect_lte(eigenFunction(new_location, two_dim_location, Phi) - 0.2352884,
+             tol)
 })
